@@ -337,6 +337,49 @@ class bugModel extends model
             if($changes) $this->action->logHistory($actionID, $changes);
         }
 
+        if($this->config->edition != 'open')
+        {
+            if($oldBug->story > 0 || $oldBug->task > 0 || $oldBug->case > 0)
+            {
+                $AID    = $oldBug->story > 0 ? ($oldBug->task > 0 ? "{$oldBug->story},{$oldBug->task}" : $oldBug->story) : $oldBug->task;
+                $AID   .= $oldBug->case  > 0 ? ",{$oldBug->case}" : '';
+                $AType  = $oldBug->story > 0 ? ($oldBug->task > 0 ? 'story,task' : 'story') : 'task';
+                $AType .= $oldBug->case  > 0 ? ",testcase" : '';
+                $this->dao->delete()->from(TABLE_RELATION)
+                    ->where('relation')->eq('generated')
+                    ->andWhere('AID')->in($AID)
+                    ->andWhere('AType')->in($AType)
+                    ->andWhere('BID')->eq($oldBug->id)
+                    ->andWhere('BType')->eq('bug')
+                    ->exec();
+            }
+            if($bug->story > 0 || $bug->task > 0 || $bug->case > 0)
+            {
+                $relation = new stdClass();
+                $relation->relation = 'generated';
+                $relation->BID      = $bug->id;
+                $relation->BType    = 'bug';
+                if($bug->story > 0)
+                {
+                    $relation->AID   = $bug->story;
+                    $relation->AType = 'story';
+                    $this->dao->replace(TABLE_RELATION)->data($relation)->exec();
+                }
+                if($bug->task > 0)
+                {
+                    $relation->AID   = $bug->task;
+                    $relation->AType = 'task';
+                    $this->dao->replace(TABLE_RELATION)->data($relation)->exec();
+                }
+                if($bug->case > 0)
+                {
+                    $relation->AID   = $bug->case;
+                    $relation->AType = 'testcase';
+                    $this->dao->replace(TABLE_RELATION)->data($relation)->exec();
+                }
+            }
+        }
+
         if(dao::isError()) return false;
 
         return $changes;
@@ -455,7 +498,7 @@ class bugModel extends model
         $files      = $this->loadModel('file')->saveUpload('bug', $bug->id);
         $fileAction = !empty($files) ? $this->lang->addFiles . implode(',', $files) . "\n" : '';
         $changes    = common::createChanges($oldBug, $bug);
-        $actionID   = $this->loadModel('action')->create('bug', $bug->id, 'Resolved', $fileAction . (!empty($bug->comment) ? $bug->comment : ''), $bug->resolution . (isset($bug->duplicateBug) ? ':' . $bug->duplicateBug : ''));
+        $actionID   = $this->loadModel('action')->create('bug', $bug->id, 'Resolved', $fileAction . (!empty($bug->comment) ? $this->post->comment : ''), $bug->resolution . (isset($bug->duplicateBug) ? ':' . $bug->duplicateBug : ''));
         if($changes) $this->action->logHistory($actionID, $changes);
 
         /* If the edition is not pms, update feedback. */
@@ -536,6 +579,7 @@ class bugModel extends model
     public function activate($bug, $kanbanParams = array())
     {
         $oldBug = parent::fetchByID($bug->id);
+        $bug    = $this->loadModel('file')->processImgURL($bug, $this->config->bug->editor->activate['id'], $this->post->uid);
 
         $this->dao->update(TABLE_BUG)->data($bug, 'comment')->check('openedBuild', 'notempty')->autoCheck()->checkFlow()->where('id')->eq($bug->id)->exec();
         if(dao::isError()) return false;
@@ -561,7 +605,7 @@ class bugModel extends model
         if($changes || $files)
         {
             $fileAction = !empty($files) ? $this->lang->addFiles . implode(',', $files) . "\n" : '';
-            $actionID   = $this->loadModel('action')->create('bug', $bug->id, 'Activated', $fileAction . $bug->comment);
+            $actionID   = $this->loadModel('action')->create('bug', $bug->id, 'Activated', $fileAction . $this->post->comment);
             $this->action->logHistory($actionID, $changes);
         }
 
@@ -580,6 +624,7 @@ class bugModel extends model
     public function close($bug, $output = array())
     {
         $oldBug = $this->getById($bug->id);
+        $bug    = $this->loadModel('file')->processImgURL($bug, $this->config->bug->editor->close['id'], $this->post->uid);
 
         $this->dao->update(TABLE_BUG)->data($bug, 'comment')->autoCheck()->checkFlow()->where('id')->eq($bug->id)->exec();
         if(dao::isError()) return false;
@@ -587,7 +632,7 @@ class bugModel extends model
         if($this->config->edition != 'open' && $oldBug->feedback) $this->loadModel('feedback')->updateStatus('bug', $oldBug->feedback, $bug->status, $oldBug->status);
 
         $changes = common::createChanges($oldBug, $bug);
-        $actionID = $this->loadModel('action')->create('bug', $bug->id, 'Closed', $bug->comment);
+        $actionID = $this->loadModel('action')->create('bug', $bug->id, 'Closed', $this->post->comment);
         if($changes) $this->action->logHistory($actionID, $changes);
 
         if($oldBug->execution)
@@ -691,6 +736,9 @@ class bugModel extends model
     public function buildSearchForm($productID, $products, $queryID, $actionURL, $branch = '0')
     {
         if(commonModel::isTutorialMode()) return;
+
+        if(empty($productID) && empty($products)) $products = $this->loadModel('product')->getPairs('', 0, '', 'all');
+
         $projectID = $this->lang->navGroup->bug == 'qa' ? 0 : $this->session->project;
 
         /* Get product params. */
@@ -721,7 +769,7 @@ class bugModel extends model
         $this->config->bug->search['params']['openedBuild']['values']   = $this->loadModel('build')->getBuildPairs(array($productID), 'all', 'withbranch|releasetag');
         $this->config->bug->search['params']['resolvedBuild']['values'] = $this->config->bug->search['params']['openedBuild']['values'];
 
-        $product = $this->loadModel('product')->fetchByID($productID);
+        $product = $this->loadModel('product')->fetchByID(empty($productID) ? key($products) : $productID);
         if($product->type == 'normal')
         {
             unset($this->config->bug->search['fields']['branch']);
@@ -749,7 +797,7 @@ class bugModel extends model
     {
         $productIdList = array();
         foreach($bugs as $bug) $productIdList[$bug->product] = $bug->product;
-        $builds = $this->loadModel('build')->getBuildPairs(array_unique($productIdList), 'all', $params = 'noterminate,nodone,hasdeleted');
+        $builds = $this->loadModel('build')->getBuildPairs(array_unique($productIdList), 'all', $params = 'hasdeleted');
 
         /* Process the openedBuild and resolvedBuild fields. */
         foreach($bugs as $bug)
@@ -2165,4 +2213,22 @@ class bugModel extends model
             ->andWhere('t3.id')->ne('')
             ->fetchGroup('revision', 'id');
     }
+
+    /**
+     * 通过任务ID获取相关的Bug
+     * @param  int         $taskID
+     *
+     * @access public
+     * @return array|false
+     */
+    public function getLinkedBugsByTaskID($taskID)
+    {
+        return $this->dao->select('t1.id,t1.title')
+            ->from(TABLE_BUG)->alias('t1')
+            ->leftJoin(TABLE_TASK)->alias('t2')->on('t1.task=t2.id')
+            ->where('t2.id')->eq($taskID)
+            ->andWhere('t1.deleted')->eq(0)
+            ->andWhere('t2.deleted')->eq(0)
+            ->fetchAll('id');
+    }
 }
