@@ -281,17 +281,15 @@ class metricModel extends model
      */
     public function getScopePairs($all = true)
     {
+        $metrics = $all ? array() : $this->metricTao->fetchMetricsByScope($this->config->metric->scopeList);
         $scopePairs = array();
-        foreach($this->config->metric->scopeList as $scope)
+        foreach($this->lang->metric->scopeList as $scope => $name)
         {
-            if(!$all)
-            {
-                $metrics = $this->metricTao->fetchMetricsByScope($scope, 1);
-                if(empty($metrics)) continue;
-            }
+            if(!$all && !isset($metrics[$scope])) continue;
+
             $scopePair = new stdclass();
             $scopePair->value = $scope;
-            $scopePair->label = $this->lang->metric->scopeList[$scope];
+            $scopePair->label = $name;
 
             $scopePairs[] = $scopePair;
         }
@@ -490,8 +488,11 @@ class metricModel extends model
      */
     public function getByCode($code, $fieldList = '*')
     {
+        static $metricsWithCode;
         if(is_array($fieldList)) $fieldList = implode(',', $fieldList);
-        return $this->dao->select($fieldList)->from(TABLE_METRIC)->where('code')->eq($code)->fetch();
+        if(!isset($metricsWithCode[$code . $fieldList])) $metricsWithCode[$code . $fieldList] = $this->fetchMetricByCode($code, $fieldList);
+
+        return $metricsWithCode[$code . $fieldList];
     }
 
     /**
@@ -505,6 +506,7 @@ class metricModel extends model
      */
     public function getByID($metricID, $fieldList = '*')
     {
+        if($metricID === 0) return false;
         if(is_array($fieldList)) $fieldList = implode(',', $fieldList);
         $metric = $this->dao->select($fieldList)->from(TABLE_METRIC)->where('id')->eq($metricID)->fetch();
 
@@ -569,7 +571,8 @@ class metricModel extends model
      */
     public function getDAO()
     {
-        if($this->config->metricDB->type == 'sqlite')
+        $dbType = $this->config->metricDB->type;
+        if($dbType === 'sqlite')
         {
             $dao = clone $this->dao;
             $dao->reset();
@@ -678,7 +681,7 @@ class metricModel extends model
      */
     public function calculateMetricByCode($code)
     {
-        $metric = $this->metricTao->fetchMetricByCode($code);
+        $metric = $this->getByCode($code);
         if(!$metric) return false;
 
         $calculator = $this->getCalculator($metric->scope, $metric->purpose, $metric->code);
@@ -710,7 +713,7 @@ class metricModel extends model
      */
     public function getResultByCodeWithArray($code, $options = array(), $type = 'realtime', $pager = null, $vision = 'rnd')
     {
-        $metric = $this->metricTao->fetchMetricByCode($code);
+        $metric = $this->getByCode($code);
         if(!$metric) return array();
         $dataFields = $this->getMetricRecordDateField($metric);
 
@@ -792,7 +795,7 @@ class metricModel extends model
     {
         if($type == 'cron')
         {
-            $metric = $this->metricTao->fetchMetricByCode($code);
+            $metric = $this->getByCode($code);
             if(!empty($metric))
             {
                 $dataFields = $this->getMetricRecordDateField($metric);
@@ -803,7 +806,7 @@ class metricModel extends model
 
         }
 
-        $metric = $this->metricTao->fetchMetricByCode($code);
+        $metric = $this->getByCode($code);
         if(!$metric) return false;
 
         $calculator = $this->getCalculator($metric->scope, $metric->purpose, $metric->code);
@@ -919,6 +922,14 @@ class metricModel extends model
     public function calculateDefaultMetric($calculator, $vision)
     {
         $statement = $this->getDataStatement($calculator, 'statement', $vision);
+        $dbType = $this->config->metricDB->type;
+        if($dbType == 'duckdb')
+        {
+            $this->loadModel('bi');
+            $sql = $statement->get();
+            $dbh = $this->app->loadDriver('duckdb');
+            $statement = $dbh->query($sql);
+        }
         $rows = $statement->fetchAll();
         if(!empty($rows)) foreach($rows as $row) $calculator->calculate($row);
     }
@@ -936,7 +947,7 @@ class metricModel extends model
      */
     public function getLatestResultByCode($code, $options = array(), $pager = null, $vision = 'rnd')
     {
-        $metric     = $this->metricTao->fetchMetricByCode($code);
+        $metric     = $this->getByCode($code);
         $dataFields = $this->getMetricRecordDateField($metric);
         $options    = $this->setDefaultOptions($options, $dataFields);
 
@@ -1041,7 +1052,7 @@ class metricModel extends model
      */
     public function getRecordFields($code)
     {
-        $metric   = $this->metricTao->fetchMetricByCode($code);
+        $metric   = $this->getByCode($code);
         $dateType = $this->getDateTypeByCode($code);
         $fields   = array($metric->scope);
 
@@ -1128,19 +1139,27 @@ class metricModel extends model
      */
     public function insertMetricLib($recordWithCode, $calcType = 'cron')
     {
-        $this->dao->begin();
         foreach($recordWithCode as $code => $records)
         {
+            $schema = '';
+            $values = array();
             foreach($records as $record)
             {
                 if(empty($record)) continue;
 
                 $record->calcType = $calcType;
                 $record->calculatedBy = $calcType == 'inference' ? $this->app->user->account : 'system';
-                $this->dao->insert(TABLE_METRICLIB)
+
+                $sql = $this->dao->insert(TABLE_METRICLIB)
                     ->data($record)
-                    ->exec();
+                    ->get();
+
+                $position = strpos($sql, 'VALUES');
+
+                if(empty($schema)) $schema .= substr($sql, 0, $position + 6);
+                $values[] = substr($sql, $position + 6);
             }
+
             $rows = count($records);
             $time = helper::now();
 
@@ -1149,8 +1168,14 @@ class metricModel extends model
                 ->set('lastCalcTime')->eq($time)
                 ->where('code')->eq($code)
                 ->exec();
+
+            while($rows > 0)
+            {
+                $sql = $schema . implode(',', array_splice($values, 0, 10000));
+                $this->dao->exec($sql);
+                $rows -= 10000;
+            }
         }
-        $this->dao->commit();
 
         return dao::isError();
     }
@@ -1597,13 +1622,12 @@ class metricModel extends model
      */
     public function processScopeList($stage = 'all')
     {
+        $metrics = $this->metricTao->fetchMetricsByScope($this->config->metric->scopeList);
         foreach($this->lang->metric->scopeList as $scope => $name)
         {
-            $metrics = $this->metricTao->fetchMetricsByScope($scope, 1);
-            if(empty($metrics))
+            if(!isset($metrics[$scope]))
             {
                 unset($this->lang->metric->scopeList[$scope]);
-
                 unset($this->lang->metric->featureBar['preview'][$scope]);
             }
         }
@@ -2510,15 +2534,20 @@ class metricModel extends model
      */
     public function getDateTypeByCode($code)
     {
-        /* Get dateType form db first. */
-        $metric = $this->getByCode($code, 'dateType');
-        if(!empty($metric->dateType)) return $metric->dateType;
+        static $dateTypes = array();
+        if(!isset($dateTypes[$code]))
+        {
+            /* Get dateType form db first. */
+            $metric = $this->getByCode($code, 'dateType');
 
-        /* Get dateType from config second. */
-        if(isset($this->config->metric->dateType[$code])) return $this->config->metric->dateType[$code];
+            if(!empty($metric->dateType)) $dateTypes[$code] = $metric->dateType;
+            /* Get dateType from config second. */
+            elseif(isset($this->config->metric->dateType[$code])) $dateTypes[$code] = $this->config->metric->dateType[$code];
+            /* Return nodate if no matches. */
+            else $dateTypes[$code] = 'nodate';
+        }
 
-        /* Return nodate if no matches. */
-        return 'nodate';
+        return $dateTypes[$code];
     }
 
     /**
